Programming 7.2 Artifact
===

### Getting Started Guide

Welcome!

This artifact is primarily a dataset. Most files can be viewed with a web
browser, text editor, or PDF reader. These files include:

- Labelled responses to our surveys
- The surveys themselves
- The coding rubric that guided our categorization

There are also a few scripts written in Racket that we used to parse and
organize data. We provide these only for archival purposes, to give a sense of
the preliminary work behind the manual analysis.

To get started:

- Open a copy of `Little-Tricky-Logic-Coded-Responses`. The main home for
  this spreadsheet is on Google Sheets at the URL below. For archival
  purposes, we include several exports (by Google) of this data:
  * Google Sheets URL: <https://docs.google.com/spreadsheets/d/15XdyeROZn9rbJDJ8piLNCwYSEqA8-6GsumJGZ14jw-E/edit?usp=sharing>
  * HTML: `Little-Tricky-Logic-Coded-Responses_html/*.html`
    - If the text in some cells is cut off or flows into the next cell,
      try to right-click the cell and inspect its source. 
  * ODS: `Little-Tricky-Logic-Coded-Responses.ods`
  * Excel: `Little-Tricky-Logic-Coded-Responses.xlsx`
  * Tab-separated text: `Little-Tricky-Logic-Coded-Responses_tsv/*.tsv`

The 11 sheets in that workbook contain the main data that supports the
conclusions in our paper.

BEWARE: Some versions of the spreadsheet may not render properly on certain
platforms. We are aware of issues with the HTML on macOS, and will add others
as we learn of them. If you find an issue, please notify us and use a different
format in the meantime.


### Overview of Claims

The paper has three major claims:

1. We find errors in the responses to three types of LTL question among four
   populations.
   - The main findings are in Table 4 and Table 5.
     * These findings are supported by the main spreadsheet
       `Little-Tricky-Logic-Coded-Responses`.
   - Minor findings are described in Section 4.
     * These findings are supported by the main spreadsheet and by
       the additional files: `round-1/output_instrument.md`,
       `round-1/output_notes.txt`, and
       `round-1/output_preliminary-misconceptions.txt`
2. We provide a code book of misconceptions.
   - Figure 4 in the paper presents the code book.
     Additionally, the files in `round-2/output_coding/` contain
     low-level outputs from the coding exercise (e.g.,
     `round-2/output_coding/coder-AB.md`) and early drafts of the
     final code book (e.g., `round-2/output_coding/history/v18.md`)
3. We provide three instruments to test for misconceptions.
   - Appendix A of the paper contains the instruments.

All claims are supported by the artifact.

The artifact is missing recordings for the Round 4 talk-alouds because
releasing these would reveal the identity of our participants.


### Step-by-Step Instructions

1. Open the main spreadsheet (Little-Tricky-Logic-Coded-Responses`).
   Check that the counts match Table 4 and Table 5 in the paper.
   Use the following descriptions to navigate the many sheets:
   - r1 EL: (not shown in paper) Round 1 English to LTL
   - r1 LE: (not shown in paper) Round 1 LTL to English
   - r2 EL: (in Table 5) Round 2 English to LTL
   - r2 LE: (in Table 5) Round 2 LTL to English
   - r2 TB: (in Table 4) Round 2 Trace Satisfaction
   - r3 EL: (in Table 5) Round 3 English to LTL
   - r3 LE: (in Table 5) Round 3 LTL to English
   - r3 TB: (in Table 4) Round 3 Trace Satisfaction
   - r4 EL: (in Table 5) Round 4 English to LTL
   - r4 LE: (in Table 5) Round 4 LTL to English
   - r4 TB: (in Table 4) Round 4 Trace Satisfaction
2. Check that the code book in the paper (Figure 5) is similar to
   the code book in `round-2/output_coding/code-book.pdf`.
3. Look over the surveys and response summaries:
   - `round-1/input_survey.txt`
   - `round-1/input_quizius-screenshots/*.png`
   - `round-2/input_survey.pdf`
   - `round-2/output_english-to-ltl.html`
   - `round-2/output_ltl-to-english.html`
   - `round-2/output_trace-sat.html`
   - `round-3/input_survey.pdf`
   - `round-3/output_report.pdf`
   - `round-4/input_survey.pdf`
   - `round-4/output_report.pdf`
4. Skim the other files.


#### Summary of Files Included

- `submission.pdf` Paper submitted to Programming 7.2
- `Little-Tricky-Logic-Coded-Responses.ods` ODS copy of main dataset
- `Little-Tricky-Logic-Coded-Responses.xlsx` Excel copy of main dataset
- `Little-Tricky-Logic-Coded-Responses/` HTML copy of main dataset
- `round-1/`
  - `context/` Lab and homework for students enrolled in the Round 1 course
  - `input_survey.txt` Text from the Quizius assignment
  - `input_quizius-screenshots` Screenshots from Quizuis (fullscreen and mobile)
  - `input_english-to-ltl.txt` Student responses to Quizius
  - `input_ltl-to-english.txt` Student responses to Quizius
  - `input_seed-questions.txt` Inputs to Quizius
  - `output_instrument.md` Preliminary instrument based on Quizius responses
  - `output_notes.txt` Notes used to inform the instrument
  - `output_preliminary-misconceptions.txt` Preliminary misconceptions based on Quizius responses

- `round-2/`
  - `input_survey.pdf` Round 2 survey
  - `input_edit.tsv` Cleaned-up survey responses
  - `mid_htmlgen.rkt` Script to generate HTML from the responses. Depends on the Forge tool to run.
  - `output_coding/` Artifacts from the codebook development process
  - `output_english-to-ltl.html` Summary of responses
  - `output_ltl-to-english.html` Summary of responses
  - `output_trace-sat.html` Summary of responses

- `round-3/`
  - `README.md` Summary of errors
  - `input_survey.pdf` Round 3 survey
  - `input_advert.txt` Email used to recruit subjects
  - `input_raw.tsv` Survey responses
  - `mid_parse.rkt` Script to print responses in spreadsheet format
  - `mid_prompt.rktd` Helper data for the parsing script
  - `output_report.pdf` Summary of responses

- `round-4/`
  - `README.md` Summary of errors
  - `input_survey.pdf` Round 4 survey
  - `input_consent-form.pdf` Consent form for participants
  - `input_edit.tsv` Cleaned-up survey responses
  - `input_raw.tsv` Survey responses
  - `mid_parser.rkt` Script to print responses in spreadsheet format
  - `output_report.pdf` Summary of responses

