#define NUM_FLOORS 5
#define NUM_USERS 3
#define UP 1
#define DOWN 2
#define IDLE 0
#define OPEN 1
#define CLOSED 0

// State - which floor's button is pushed, 
bool button[NUM_FLOORS];
int motion = IDLE;
byte floor = 0;
bool door = CLOSED;

proctype safe_live_elevator(){

    // Elevator controller manages by moving to 
    // the top and then to the lowest floor
    // should be safe and live, but makes pointless trips

    motion = UP; // initially moving up
    do
    // The elevator moving up.
    ::  motion == UP ->
    assert(floor >= 0 && floor < NUM_FLOORS);
    printf("%d %d: %d %d %d %d %d\n", floor, motion, button[0], button[1], button[2], button[3], button[4]);
        if
        // Passing a floor with a button pushed
        ::  button[floor] ->
            motion = IDLE;
            door = 1;
            button[floor] = 0;
            door = 0;
        ::  !button[floor] ->
            skip;
        fi;
        motion = UP;
        // move up a floor or change the direction
        if
        ::  floor == NUM_FLOORS - 1 -> motion = DOWN; 
        ::  floor < NUM_FLOORS-1 -> floor = floor + 1;
        fi;
    // The elevator moving down.
    ::  motion == DOWN ->
    assert(floor >= 0 && floor < NUM_FLOORS);
    printf("%d %d: %d %d %d %d %d\n", floor, motion, button[0], button[1], button[2], button[3], button[4]);
        if
        // Passing a floor with a button pushed
        ::  button[floor] ->
            motion = IDLE;
            door = 1;
            button[floor] = 0;
            door = 0;
        ::  !button[floor] ->
            skip;
        fi;
        motion = DOWN;
        // move down a floor or change the direction
        if
        ::  floor == 0 -> motion = UP; 
        ::  floor > 0 -> floor = floor - 1;
        fi;
    od;
}

proctype min_max_elevator() {

  // elevator principle algorithm
  // alternates going to the highest request and 
  // going to the lowest request
  // picks up every request that it passes along the way
  // should satisfy all 3 properties

  int toFloor;
  int direction = UP;
  do
      :: 
      toFloor = -1;
      int i;
      if
        :: direction == UP ->
          //find highest floor
          i = ( NUM_FLOORS - 1);
          do 
            :: i >= floor ->
              if 
                :: button[i] == 1 ->
                  toFloor = i;
                  break;
                :: else -> skip;
              fi;
              i--;
            :: else -> break;
          od;
        :: direction == DOWN ->
          //find lowest floor
          for (i : 0 .. floor) {
            if 
              :: button[i] == 1 ->
                toFloor = i;
                break;
              :: else -> skip;
            fi;
          }
      fi;

      printf("direction: %d, current: %d, target: %d\n", direction, 
        floor, toFloor);

      do
        :: toFloor == -1 -> break;
        :: else ->
          if 
            :: button[floor] == true ->
              door = OPEN; // open door
              button[floor] = false;
              printf("picked up at floor %d, now %d\n", 
                floor, button[floor]);
              door = CLOSED; // close door
            :: else -> skip;
          fi;

          if
            :: toFloor == floor ->
              break;
            :: else ->
              if
                :: direction == UP ->
                  motion = UP;
                  floor = floor + 1;
                  motion = IDLE;
                :: direction == DOWN ->
                  motion = DOWN;
                  floor = floor - 1;
                  motion = IDLE;
              fi;
          fi;
      od;

      //switch direction each round
      if
        :: direction == UP ->
          direction = DOWN;
        :: direction == DOWN ->
          direction = UP;
      fi;
  od;
}

active proctype simple_elevator() {
  // moves the elevator to the lowest floor with a pending request
  // not careful about how it opens and closes the doors
  // not safe or live, but no pointless trips

  int lowestFloor;
  do
    :: lowestFloor = -1;
        int i;
      for (i : 0 .. ( NUM_FLOORS - 1)) {
        if 
          :: button[i] == true ->
            lowestFloor = i;
            break;
          :: else -> skip;
        fi;
      }

        do
          :: lowestFloor == -1 -> break;
          :: else ->
            if
              :: lowestFloor > floor ->
                motion = UP;
                floor++;
              :: lowestFloor < floor ->
                motion = DOWN;
                floor--;
              :: lowestFloor == floor ->
                door = OPEN;  
                motion = IDLE;
                button[lowestFloor] = false;
                printf("picked up at floor %d, now %d\n", lowestFloor, 
                  button[lowestFloor]);
                door = CLOSED;
                break;
            fi;
        od;
    od;
}

active [NUM_USERS] proctype user() {
  int req;
  do
    ::  
        // non-deterministically pick a random floor 
        req = 0; 
        do
        :: req < (NUM_FLOORS - 1) -> req++;
        :: break;
        od;

        button[req] = 1;
        printf("requested floor: %d\n", req);
        !button[req];
  od;
}

// If the door is open the motion is IDLE.
ltl door_safety {
  always(door == 1 implies motion == IDLE)
}

// If a button is pressed the elevator will eventually reach that floor.
ltl non_starvation {
  always(button[NUM_FLOORS - 1] implies eventually (floor == NUM_FLOORS-1 
    && door == OPEN))
}

ltl no_pointless_trips {
  always(
    ((motion == UP && floor == 4) implies button[4]) &&
    ((motion == UP && floor == 3) implies (button[3] || button[4])) &&
    ((motion == UP && floor == 2) implies 
      (button[2] || button[3] || button[4])) &&
    ((motion == UP && floor == 1) implies 
    (button[1] || button[2] || button[3] || button[4])) &&
    ((motion == UP && floor == 0) implies 
      (button[0] || button[1] || button[2] || button[3] || button[4])) &&

    ((motion == DOWN && floor == 0) implies button[0]) &&
    ((motion == DOWN && floor == 1) implies (button[0] || button[1])) &&
    ((motion == DOWN && floor == 2) implies 
      (button[0] || button[1] || button[2])) &&
    ((motion == DOWN && floor == 3) implies 
    (button[0] || button[1] || button[2] || button[3])) &&
    ((motion == DOWN && floor == 4) implies 
      (button[0] || button[1] || button[2] || button[3] || button[4]))  
  );
}