/*
Dining smithosophers problem with 'polite dining' algorithm
Used d_step to shorten the run time by not blowing up the number of states.
*/

#define NUM_SMITH 5 //This is also num_chopstick
#define left(p) p
#define right(p) (p + 1)%NUM_SMITH
#define NONE 0
#define LEFT 1
#define RIGHT 2
#define BOTH 3
#define CLEAN 0
#define DIRTY 1
#define FOR(x) for(i: 0 .. x-1)


bool chopState[NUM_SMITH];
byte smithState[NUM_SMITH];
bool requests[NUM_SMITH];

inline LeftOk(smith, res){
	// Check whether it is possible for blacksmith 'smith' to take their left chopstick.
    // Store the bool value into res.
	res = res || (smithState[smith] != BOTH && smithState[smith] != LEFT && requests[left(smith)] == 0);
}

inline RightOk(smith, res){
	// Check whether it is possible for blacksmith 'smith' to take their right chopstick.
    // Store the bool value into res.
	res = res || (smithState[smith] != BOTH && smithState[smith] != RIGHT && requests[right(smith)] == 0);
}

inline RequestOk(chop, res){
	// Check whether it is possible for chopstick 'chop' to be cleaned and change the owners.
    // Store the bool value into res.
	res = res || (chopState[chop] == DIRTY && requests[chop] == 1);
}

inline DirtyOk(smith, res){
	// Check whether it is possible for blacksmith 'smith' to drop their chopsticks.
    // Store the bool value into res.
	res = res || (smithState[smith] == BOTH && chopState[left(smith)] == CLEAN && chopState[right(smith)] == CLEAN);
}

inline RequestLeft(){
	d_step{

		byte smith = 255; 
		// Non-deterministically chooses a random blacksmith who can request their left chopstick. 
	    // Stores the results in smith.
		bool ok = 0 ; byte tmp = 0; 
		do 
	    :: tmp < NUM_SMITH -> 
	    	ok = 0;  
	    	LeftOk(tmp, ok); 
	    	if 
	    	:: ok -> smith = tmp; 
	    	:: else -> skip; 
	    	fi; 
	    	tmp++; 
	    :: smith != 255 -> break;  
	    od;
	    // Update the states to reflect 'smith' requesting their left chopstick
		printf("smith %d requested chopstick %d LEFT\n", smith, left(smith));
		requests[left(smith)] = 1;

	}
}

inline RequestRight(){
	d_step{
		byte smith = 255; 
		// Non-deterministically chooses a random blacksmith who can request their left chopstick. 
	    // Stores the results in smith.
		bool ok = 0 ; byte tmp = 0; 
		do 
	    :: tmp < NUM_SMITH -> 
	    	ok = 0;  
	    	RightOk(tmp, ok); 
	    	if 
	    	:: ok -> smith = tmp; 
	    	:: else -> skip; 
	    	fi; 
	    	tmp++; 
	    :: smith != 255 -> break;  
	    od;

    	// Update the states to reflect 'smith' requesting their right chopstick
	    printf("smith %d requested chopstick %d RIGHT\n", smith, right(smith));
		requests[right(smith)] = 1;
	}
}

inline DirtyChops(){
	d_step{
		byte smith = 255; 
		bool ok = 0 ; byte tmp = 0;
		// Non-deterministically chooses a random blacksmith who use both of their chopsticks
		// (these chopsticks should be clean). 
	    // Stores the results in smith.
 
		do 
	    :: tmp < NUM_SMITH -> 
	    	ok = 0;  
	    	DirtyOk(tmp, ok); 
	    	if 
	    	:: ok -> smith = tmp; 
	    	:: else -> skip; 
	    	fi; 
	    	tmp++; 
	    :: smith != 255 -> break;  
	    od;

	    // Update the states to reflect 'smith' using their chopsticks to eat leaving them dirty
		printf("smith %d drop chopsticks %d %d\n", smith, right(smith), left(smith));
		chopState[right(smith)] = DIRTY;
		chopState[left(smith)] = DIRTY;
	}
}

inline HonorRequest(){
	d_step{
		byte chop = 255; 
		// Non-deterministically chooses a random 'chopstick' which is currently dirty. 
	    // Stores the results in chop.
		bool ok = 0 ; byte tmp = 0; 
		do 
	    :: tmp < NUM_SMITH -> 
	    	ok = 0;  
	    	RequestOk(tmp, ok); 
	    	if 
	    	:: ok -> chop = tmp; 
	    	:: else -> skip; 
	    	fi; 
	    	tmp++; 
	    :: chop != 255 -> break;  
	    od;
	    // Update the states to reflect a chopstick 'chop' being washed and transfered to another blacksmith.
		printf("Chopstick %d is washed and changed the owner.\n", chop);
		chopState[chop] = CLEAN;
		requests[chop] = 0;
		byte own = chop, requ = (chop - 1 + NUM_SMITH) % NUM_SMITH;
		printf("Left: %d %d\n", own, smithState[own]);
		printf("Right: %d %d\n", requ, smithState[requ]);
		smithState[own] = smithState[own] ^ LEFT;
		smithState[requ] = smithState[requ] ^ RIGHT;
		printf("Left: %d %d\n", own, smithState[own]);
		printf("Right: %d %d\n", requ, smithState[requ]);
	}
}

active proctype main(){
	int i; 
	FOR(NUM_SMITH){
		chopState[i] = DIRTY;
		if
		:: i == 0 -> smithState[i] = BOTH;
		:: i == NUM_SMITH-1 -> smithState[i] = NONE;
		:: else -> smithState[i] = RIGHT;
		fi;
	}

	bool okL, okR, okD, okH;
	do
	:: 
		okL = 0; okR = 0; okD = 0; okH = 0;
		FOR(NUM_SMITH){
			LeftOk(i, okL);
			RightOk(i, okR);
			DirtyOk(i, okD);
			RequestOk(i, okH);
		}
		if
		::	okL -> RequestLeft();
		::  okR -> RequestRight();
		::  okD -> DirtyChops();
		::  okH -> HonorRequest(); 
		fi;
	od;
}

#define noStarv(p) always eventually (smithState[p] == BOTH) 
ltl non_starvation{
	noStarv(0) && noStarv(1) && noStarv(2) && noStarv(3) 
}