/*
Dining blacksmiths
*/

#define NUM_SMITH 5 // This is also the number of chopsticks
#define left(p) p
#define right(p) (p + 1)%NUM_SMITH
#define NONE 0
#define LEFT 1
#define RIGHT 2
#define BOTH 3
#define AVAILABLE 0
#define TAKEN 1

bool chopState[NUM_SMITH];
byte smithState[NUM_SMITH];

inline LeftOk(smith, res) {

    // Check whether it is possible for blacksmith 'smith' to take their left chopstick.
    // Store the bool value into res.

    res = (chopState[left(smith)] == AVAILABLE);
}

inline RightOk(smith, res) {

    // Check whether it is possible for blacksmith 'smith' to take their right chopstick.
    // Store the bool value into res.

    res = (chopState[right(smith)] == AVAILABLE);

}

inline DropOk(smith, res) {
    // TODO: Fill in

    // Check whether it is possible for blacksmith 'smith' to drop their chopsticks.
    // Store the bool value into res.

    res = (smithState[smith] == BOTH);

}

inline TakeLeft() {

    // Non-deterministically chooses a random blacksmith who can take their left chopstick. 
    // Stores the results in smith.
    byte smith = 255;
    byte tmp = 0; 
    bool ok = 0 ;  
    do
    :: tmp < (NUM_SMITH) ->
        ok = 0;
        LeftOk(tmp, ok);
        if
        :: ok -> smith = tmp;
        :: else -> skip;
        fi;
        tmp++;
    :: smith != 255 -> break; 
    od;

    printf("Blacksmith %d took chopstick %d LEFT\n", smith, left(smith));
    //////////////////////////////////////////////////////////

    // Update the states to reflect smith taking their left chopstick

    if 
        :: smithState[smith] == NONE ->
            smithState[smith] = LEFT;
        :: smithState[smith] == RIGHT ->
            smithState[smith] = BOTH
        // shouldn't need the 3rd left case bc chopstick is available
    fi;
    chopState[left(smith)] = TAKEN;

}

inline TakeRight() {

    // Non-deterministically chooses a random blacksmith who can take their right chopstick. 
    // Stores the results in smith.
    byte smith = 255;
    byte tmp = 0; 
    bool ok = 0 ;  
    do
    :: tmp < (NUM_SMITH) ->
        ok = 0;
        RightOk(tmp, ok);
        if
        :: ok -> smith = tmp;
        :: else -> skip;
        fi;
        tmp++;
    :: smith != 255 -> break; 
    od;

    printf("Blacksmith %d took chopstick %d RIGHT\n", smith, right(smith));

    // Update the states to reflect smith taking their right chopstick

    if 
        :: smithState[smith] == NONE ->
            smithState[smith] = RIGHT;
        :: smithState[smith] == LEFT ->
            smithState[smith] = BOTH
        // shouldn't need the 3rd left case bc chopstick is available
    fi;
    chopState[right(smith)] = TAKEN;

}

inline DropChops() {
    // Non-deterministically chooses a random blacksmith who can drop their chopsticks. 
    // Stores the results in smith.
    byte smith = 255;
    byte tmp = 0; 
    bool ok = 0 ;  
    do
    :: tmp < (NUM_SMITH) ->
        ok = 0;
        DropOk(tmp, ok);
        if
        :: ok -> smith = tmp;
        :: else -> skip;
        fi;
        tmp++;
    :: smith != 255 -> break; 
    od;

    printf("Blacksmith %d dropped their chopsticks %d and %d.\n", smith, left(smith), right(smith));

    smithState[smith] = NONE;
    chopState[left(smith)] = AVAILABLE;
    chopState[right(smith)] = AVAILABLE;

}

active proctype main() {
    bool okL, okR, okD, w;
    int i; 
    do
    :: 
        okL = 0; okR = 0; okD = 0;
        for (i : 0 .. ( NUM_SMITH - 1)) {
            LeftOk(i, okL);
            RightOk(i, okR);
            DropOk(i, okD);
        }
        if
        ::  okL ->  TakeLeft();
        ::  okR ->  TakeRight();
        ::  okD ->  DropChops();
        fi;
    od;
}

ltl non_starvation {
    always (
        eventually(smithState[0] == BOTH) &&
        eventually(smithState[1] == BOTH) &&
        eventually(smithState[2] == BOTH) &&
        eventually(smithState[3] == BOTH) &&
        eventually(smithState[4] == BOTH)
    );
}